WebServer wserver(80);

/*
 * Function to handle unknown URLs
 */
void handleNotFound() {
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += wserver.uri();
  message += "\nMethod: ";
  message += (wserver.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += wserver.args();
  message += "\n";
  for (uint8_t i = 0; i < wserver.args(); i++) {
    message += " " + wserver.argName(i) + ": " + wserver.arg(i) + "\n";
  }
  wserver.send(404, "text/plain", message);
}

/*
 * Function for writing WiFi creds to EEPROM
 * Returns: true if save successful, false if unsuccessful
 */
bool writeToMemory(String ssid, String pass, String ip, String sn, String gw) {
  char buff1[31];
  char buff2[31];
  char buff3[16];
  char buff4[16];
  char buff5[16];

  ssid.toCharArray(buff1, 31);
  pass.toCharArray(buff2, 31);
  ip.toCharArray(buff3, 16);
  sn.toCharArray(buff4, 16);
  gw.toCharArray(buff5, 16);

  EEPROM.writeString(0, buff1);
  EEPROM.writeString(31, buff2);
  EEPROM.writeString(62, buff3);
  EEPROM.writeString(78, buff4);
  EEPROM.writeString(94, buff5);
  delay(100);
  String s = EEPROM.readString(0);
  String p = EEPROM.readString(31);
  String i = EEPROM.readString(62);
  String n = EEPROM.readString(78);
  String g = EEPROM.readString(94);
  //#if DEBUG
  Serial.print("Stored SSID, password, IP, are: ");
  Serial.print(s);
  Serial.print(" / ");
  Serial.print(p);
  Serial.print(" / ");
  Serial.print(i);
  Serial.print("/");
  Serial.print(n);
  Serial.print("/");
  Serial.print(g);
  //#endif
  if (ssid == s && pass == p && ip == i) {
    return true;
  } else {
    return false;
  }
}


/*
 * Function for handling form
 */
void handleSubmit() {
  String response_success = "<h1>Success</h1>";
  response_success += "<h2>Device will restart in 3 seconds</h2>";

  String response_error = "<h1>Error</h1>";
  response_error += "<h2><a href='/'>Go back</a>to try again";

  if (writeToMemory(String(wserver.arg("ssid")), String(wserver.arg("password")), String(wserver.arg("ip")), String(wserver.arg("sn")), String(wserver.arg("gw")))) {
    wserver.send(200, "text/html", response_success);
    EEPROM.commit();
    delay(3000);
    ESP.restart();
  } else {
    wserver.send(200, "text/html", response_error);
  }
}

/*
 * Function for home page
 */
void handleRoot() {
  if (wserver.hasArg("ssid") && wserver.hasArg("password") && wserver.hasArg("ip")) {
    handleSubmit();
  } else {
    wserver.send(200, "text/html", INDEX_HTML);
  }
}

/*
 * Function for loading form
 * Returns: false if no WiFi creds in EEPROM
 */
bool loadWIFICredsForm() {
  String s = EEPROM.readString(0);
  String p = EEPROM.readString(31);
  String i = EEPROM.readString(62);
  String sn = EEPROM.readString(78);
  String gw = EEPROM.readString(94);

  const char* ssid = "RFXCOM WiFi Manager";
  const char* password = "12345678";

  Serial.println("Setting Access Point...");

  WiFi.softAP(ssid, password);

  IPAddress IP = WiFi.softAPIP();

  Serial.print("AP IP address: ");
  Serial.println(IP);

  wserver.on("/", handleRoot);

  wserver.onNotFound(handleNotFound);

  wserver.begin();

  Serial.println("HTTP server started");

  while (s.length() <= 0 && p.length() <= 0) {
    wserver.handleClient();
    delay(100);
  }

  return false;
}